import { IProduct } from '@/interfaces/IProduct.interface';
import path from 'path';
import * as fs from 'fs';
import { ProductDto } from '@/dto/product.dto';

const filePath = path.join(__dirname, '../../data');

export class ProductService {
  private products: IProduct[] = [];

  constructor() {
    this.init();
  }

  init(): void {
    try {
      const fileContent = fs.readFileSync(`${filePath}/products.json`);
      this.products = JSON.parse(fileContent.toString());
    } catch (e) {
      this.products = [];
    }
  }

  save(): void {
    fs.writeFileSync(`${filePath}/products.json`, JSON.stringify(this.products, null, 2));
  }

  getAllProducts = (): IProduct[] => {
    return this.products;
  };

  getProduct = (id: string): IProduct => {
    const product = this.products.find((product) => product.id === id);
    if (product) {
      return product;
    }
    throw new Error('Invalid id');
  };

  createProduct = (data: ProductDto): IProduct => {
    const product = {
      ...data,
      id: Math.random().toString()
    };
    this.products.push(product);
    this.save();
    return product;
  };
}
